--==============================================================================
-- CERN (BE-CO-HT)
-- Testbench for CONV-TTL-BLO design
--==============================================================================
--
-- author: Theodor Stana (t.stana@cern.ch)
--
-- date of creation: 2014-02-18
--
-- version: 1.0
--
-- description:
--    Design-wide simulation testbench for the CONV-TTL-BLO gateware. Currently
--    simulated features include:
--      - pulse triggering
--
-- dependencies:
--    None.
--
--==============================================================================
-- GNU LESSER GENERAL PUBLIC LICENSE
--==============================================================================
-- This source file is free software; you can redistribute it and/or modify it
-- under the terms of the GNU Lesser General Public License as published by the
-- Free Software Foundation; either version 2.1 of the License, or (at your
-- option) any later version. This source is distributed in the hope that it
-- will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-- See the GNU Lesser General Public License for more details. You should have
-- received a copy of the GNU Lesser General Public License along with this
-- source; if not, download it from http://www.gnu.org/licenses/lgpl-2.1.html
--==============================================================================
-- last changes:
--    2014-02-18   Theodor Stana     File created
--==============================================================================
-- TODO: -
--==============================================================================

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;


entity testbench is
end entity testbench;


architecture behav of testbench is

  --============================================================================
  -- Type declarations
  --============================================================================

  --============================================================================
  -- Constant declarations
  --============================================================================
  constant c_clk_20_per   : time := 50 ns;
  constant c_clk_125_per  : time :=  8 ns;

  --============================================================================
  -- Component declarations
  --============================================================================
  component conv_ttl_blo is
    generic
    (
      g_nr_ttl_chan : natural := 6;
      g_nr_inv_chan : natural := 4
    );
    port
    (
      -- Clocks
      -- 20 MHz from VCXO
      clk_20_vcxo_i           : in  std_logic;
      -- 125 MHz from clock generator
      fpga_clk_p_i           : in  std_logic;
      fpga_clk_n_i           : in  std_logic;
  
      -- LEDs
      led_ctrl0_o            : out std_logic;
      led_ctrl0_oen_o        : out std_logic;
      led_ctrl1_o            : out std_logic;
      led_ctrl1_oen_o        : out std_logic;
      led_multicast_2_0_o    : out std_logic;
      led_multicast_3_1_o    : out std_logic;
      led_wr_gmt_ttl_ttln_o  : out std_logic;
      led_wr_link_syserror_o : out std_logic;
      led_wr_ok_syspw_o      : out std_logic;
      led_wr_ownaddr_i2c_o   : out std_logic;
  
      -- I/Os for pulses
      pulse_front_led_n_o    : out std_logic_vector(g_nr_ttl_chan downto 1);
      pulse_rear_led_n_o     : out std_logic_vector(g_nr_ttl_chan downto 1);
      fpga_input_ttl_n_i     : in  std_logic_vector(g_nr_ttl_chan downto 1);
      fpga_out_ttl_o         : out std_logic_vector(g_nr_ttl_chan downto 1);
      fpga_blo_in_i          : in  std_logic_vector(g_nr_ttl_chan downto 1);
      fpga_trig_blo_o        : out std_logic_vector(g_nr_ttl_chan downto 1);
      inv_in_n_i             : in  std_logic_vector(g_nr_inv_chan downto 1);
      inv_out_o              : out std_logic_vector(g_nr_inv_chan downto 1);
  
      -- Output enable lines
      fpga_oe_o              : out std_logic;
      fpga_blo_oe_o          : out std_logic;
      fpga_trig_ttl_oe_o     : out std_logic;
      fpga_inv_oe_o          : out std_logic;
  
      --TTL/INV_TTL_N
      ttl_switch_n_i         : in  std_logic;
  
      extra_switch_n_i       : in  std_logic_vector(7 downto 1);
  
      -- Lines for the i2c_slave
      scl_i                  : in  std_logic;
      scl_o                  : out std_logic;
      scl_oe_o               : out std_logic;
      sda_i                  : in  std_logic;
      sda_o                  : out std_logic;
      sda_oe_o               : out std_logic;
      fpga_ga_i              : in  std_logic_vector(4 downto 0);
      fpga_gap_i             : in  std_logic;
  
      -- Flash memory lines
      fpga_prom_cclk_o       : out std_logic;
      fpga_prom_cso_b_n_o    : out std_logic;
      fpga_prom_mosi_o       : out std_logic;
      fpga_prom_miso_i       : in  std_logic;
  
      -- Blocking power supply reset line
      mr_n_o                 : out std_logic;
  
      -- Thermometer line
      thermometer_b          : inout std_logic;
  
      -- PLL DACs
      -- DAC1: 20 MHz VCXO control
      fpga_plldac1_din_o     : out std_logic;
      fpga_plldac1_sclk_o    : out std_logic;
      fpga_plldac1_sync_n_o  : out std_logic;
      -- DAC2: 125 MHz clock generator control
      fpga_plldac2_din_o     : out std_logic;
      fpga_plldac2_sclk_o    : out std_logic;
      fpga_plldac2_sync_n_o  : out std_logic;
  
      -- SFP lines
      fpga_sfp_los_i         : in    std_logic;
      fpga_sfp_mod_def0_i    : in    std_logic;
      fpga_sfp_rate_select_o : out   std_logic;
      fpga_sfp_mod_def1_b    : inout std_logic;
      fpga_sfp_mod_def2_b    : inout std_logic;
      fpga_sfp_tx_disable_o  : out   std_logic;
      fpga_sfp_tx_fault_i    : in    std_logic;
  
      -- RTM identifiers, should match with the expected values
      fpga_rtmm_n_i          : in std_logic_vector(2 downto 0);
      fpga_rtmp_n_i          : in std_logic_vector(2 downto 0)
    );
  end component conv_ttl_blo;

  --============================================================================
  -- Signal declarations
  --============================================================================
  signal clk_20, clk_125              : std_logic;
  signal clk_125_p, clk_125_n         : std_logic;
  signal rst_n                        : std_logic;

  signal pulse_led_front_n            : std_logic_vector(6 downto 1);
  signal pulse_led_front              : std_logic_vector(6 downto 1);
  signal pulse_led_rear_n             : std_logic_vector(6 downto 1);
  signal pulse_led_rear               : std_logic_vector(6 downto 1);
  signal ttl_inp_n, ttl_outp          : std_logic_vector(6 downto 1);
  signal ttl_pulse                    : std_logic_vector(6 downto 1);
  signal inv_pulse                    : std_logic_vector(6 downto 1);
  signal blo_inp, blo_outp            : std_logic_vector(6 downto 1);
  signal blo_pulse                    : std_logic_vector(6 downto 1);
  signal oe, blo_oe, ttl_oe, inv_oe   : std_logic;
  signal ttl_switch_n                 : std_logic;
  signal switches_n                   : std_logic_vector(7 downto 1);

--==============================================================================
--  architecture begin
--==============================================================================
begin

  --============================================================================
  -- Generate clock signals
  --============================================================================
  p_clk_20 : process
  begin
    clk_20 <= '0';
    wait for c_clk_20_per/2;
    clk_20 <= '1';
    wait for c_clk_20_per/2;
  end process;

  p_clk_125 : process
  begin
    clk_125 <= '0';
    wait for c_clk_125_per/2;
    clk_125 <= '1';
    wait for c_clk_125_per/2;
  end process;

  clk_125_p <= clk_125;
  clk_125_n <= not clk_125;

  --============================================================================
  -- Instantiate the DUT
  --============================================================================
  cmp_dut : conv_ttl_blo
    generic map
    (
      g_nr_ttl_chan => 6,
      g_nr_inv_chan => 4
    )
    port map
    (
      -- Clocks
      -- 20 MHz from VCXO
      clk_20_vcxo_i          => clk_20,
      -- 125 MHz from clock generator
      fpga_clk_p_i           => clk_125_p,
      fpga_clk_n_i           => clk_125_n,

      -- LEDs
      led_ctrl0_o            => open,
      led_ctrl0_oen_o        => open,
      led_ctrl1_o            => open,
      led_ctrl1_oen_o        => open,
      led_multicast_2_0_o    => open,
      led_multicast_3_1_o    => open,
      led_wr_gmt_ttl_ttln_o  => open,
      led_wr_link_syserror_o => open,
      led_wr_ok_syspw_o      => open,
      led_wr_ownaddr_i2c_o   => open,

      -- I/Os for pulses
      pulse_front_led_n_o    => pulse_led_front_n,
      pulse_rear_led_n_o     => pulse_led_rear_n,
      fpga_input_ttl_n_i     => ttl_inp_n,
      fpga_out_ttl_o         => ttl_outp,
      fpga_blo_in_i          => blo_inp,
      fpga_trig_blo_o        => blo_outp,
      inv_in_n_i             => (others => '1'),
      inv_out_o              => open,

      -- Output enable lines
      fpga_oe_o              => oe,
      fpga_blo_oe_o          => blo_oe,
      fpga_trig_ttl_oe_o     => ttl_oe,
      fpga_inv_oe_o          => inv_oe,

      --TTL/INV_TTL_N
      ttl_switch_n_i         => ttl_switch_n,

      extra_switch_n_i       => switches_n,

      -- Lines for the i2c_slave
      scl_i                  => '1',
      scl_o                  => open,
      scl_oe_o               => open,
      sda_i                  => '1',
      sda_o                  => open,
      sda_oe_o               => open,
      fpga_ga_i              => (others => '1'),
      fpga_gap_i             => '0',

      -- Flash memory lines
      fpga_prom_cclk_o       => open,
      fpga_prom_cso_b_n_o    => open,
      fpga_prom_mosi_o       => open,
      fpga_prom_miso_i       => 'Z',

      -- Blocking power supply reset line
      mr_n_o                 => rst_n,

      -- Thermometer line
      thermometer_b          => open,

      -- PLL DACs
      -- DAC1: 20 MHz VCXO control
      fpga_plldac1_din_o     => open,
      fpga_plldac1_sclk_o    => open,
      fpga_plldac1_sync_n_o  => open,
      -- DAC2: 125 MHz clock generator control
      fpga_plldac2_din_o     => open,
      fpga_plldac2_sclk_o    => open,
      fpga_plldac2_sync_n_o  => open,

      -- SFP lines
      fpga_sfp_los_i         => '1',
      fpga_sfp_mod_def0_i    => '1',
      fpga_sfp_rate_select_o => open,
      fpga_sfp_mod_def1_b    => open,
      fpga_sfp_mod_def2_b    => open,
      fpga_sfp_tx_disable_o  => open,
      fpga_sfp_tx_fault_i    => '1',

      -- RTM identifiers, should match with the expected values
      fpga_rtmm_n_i          => (others => '0'),
      fpga_rtmp_n_i          => (others => '0')
    );

  --============================================================================
  -- Pulse outputs assignment based on OE signals
  --============================================================================
  ttl_pulse <= ttl_outp when oe = '1' and ttl_oe = '1' else (others => '0');
  blo_pulse <= blo_outp when oe = '1' and blo_oe = '1' else (others => '0');
  inv_pulse <= blo_outp when oe = '1' and inv_oe = '1' else (others => '0');

  --============================================================================
  -- Switches
  --============================================================================
  -- TTL
  ttl_switch_n <= '0';
  -- GF
  switches_n(1) <= '0';
  -- other
  switches_n(7 downto 2) <= (others => '1');

  --============================================================================
  -- Pulse LEDs
  --============================================================================
  pulse_led_front <= not pulse_led_front_n;
  pulse_led_rear  <= not pulse_led_rear_n;

  --============================================================================
  -- Pulse stimuli
  --============================================================================
  blo_inp <= (others => '0');
  ttl_inp_n(5 downto 1) <= (others => '1');

  p_ttl_stim : process
  begin
    ttl_inp_n(6) <= '1';
    wait for 2.5 us;
    ttl_inp_n(6) <= '0';
    wait for 1.2 us;
    ttl_inp_n(6) <= '1';
    for i in 1 to 200 loop
      wait for 10 us;
      ttl_inp_n(6) <= '0';
      wait for 500 ns;
      ttl_inp_n(6) <= '1';
    end loop;
    wait;
  end process p_ttl_stim;

end architecture behav;
--==============================================================================
--  architecture end
--==============================================================================
