--==============================================================================
-- CERN (BE-CO-HT)
-- Testbench for pulse time-tagging core
--==============================================================================
--
-- author: Theodor Stana (t.stana@cern.ch)
--
-- date of creation: 2014-02-06
--
-- version: 1.0
--
-- description:
--
-- dependencies:
--
-- references:
--
--==============================================================================
-- GNU LESSER GENERAL PUBLIC LICENSE
--==============================================================================
-- This source file is free software; you can redistribute it and/or modify it
-- under the terms of the GNU Lesser General Public License as published by the
-- Free Software Foundation; either version 2.1 of the License, or (at your
-- option) any later version. This source is distributed in the hope that it
-- will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-- See the GNU Lesser General Public License for more details. You should have
-- received a copy of the GNU Lesser General Public License along with this
-- source; if not, download it from http://www.gnu.org/licenses/lgpl-2.1.html
--==============================================================================
-- last changes:
--    2014-02-06   Theodor Stana     File created
--==============================================================================
-- TODO: -
--==============================================================================

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.gencores_pkg.all;

entity testbench is
end entity testbench;


architecture behav of testbench is

  --============================================================================
  -- Type declarations
  --============================================================================
  
  --============================================================================
  -- Constant declarations
  --============================================================================
  constant c_clk_20_per  : time := 50 ns;
  constant c_clk_125_per : time := 8 ns;
  constant c_reset_width : time := 31 ns;

  --============================================================================
  -- Component declarations
  --============================================================================
  -- DUT component
  component pulse_timetag is
    generic
    (
    -- Frequency in Hz of the clk_i signal
      g_clk_rate : positive := 125000000;
    -- Number of repetition channels
      g_nr_chan  : positive := 6
    );
    port
    (
    -- Clock and active-low reset
      clk_i            : in  std_logic;
      rst_n_i          : in  std_logic;

    -- Asynchronous pulse input
      pulse_a_i        : in  std_logic_vector(g_nr_chan-1 downto 0);

    -- Time inputs from White Rabbit
      wr_tm_cycles_i   : in  std_logic_vector(27 downto 0);
      wr_tm_tai_i      : in  std_logic_vector(39 downto 0);
      wr_tm_valid_i    : in  std_logic;

    -- Timing inputs from Wishbone-mapped registers
      wb_tm_tai_l_i    : in  std_logic_vector(31 downto 0);
      wb_tm_tai_l_ld_i : in  std_logic;
      wb_tm_tai_h_i    : in  std_logic_vector( 7 downto 0);
      wb_tm_tai_h_ld_i : in  std_logic;

    -- Timing outputs
      tm_cycles_o      : out std_logic_vector(27 downto 0);
      tm_tai_o         : out std_logic_vector(39 downto 0)
    );
  end component pulse_timetag;

  --============================================================================
  -- Signal declarations
  --============================================================================
  -- Clock, reset signals
  signal clk_20, clk_125     : std_logic;
  signal rst_20_n, rst_125_n : std_logic;

  -- Register signals to/from "Wishbone"
  signal tvlr_ld, tvhr_ld : std_logic;
  signal ldh, ldl         : std_logic;
  signal tvlr, l          : std_logic_vector(31 downto 0);
  signal tvhr, h          : std_logic_vector(7 downto 0);

  -- Cycles, TAI and tick to "Wishbone"
  signal tm_cycles   : std_logic_vector(27 downto 0);
  signal tm_tai      : std_logic_vector(39 downto 0);

--==============================================================================
--  architecture begin
--==============================================================================
begin

  --============================================================================
  -- DUT instantiation
  --============================================================================
  cmp_dut : pulse_timetag
    generic map
    (
      g_clk_rate => 125000000,
      g_nr_chan  => 6
    )
    port map
    (
      clk_i            => clk_125,
      rst_n_i          => rst_125_n,

      pulse_a_i        => (others => '0'),

      wr_tm_cycles_i   => (others => '0'),
      wr_tm_tai_i      => (others => '0'),
      wr_tm_valid_i    => '0',

      wb_tm_tai_l_i    => tvlr,
      wb_tm_tai_l_ld_i => tvlr_ld,
      wb_tm_tai_h_i    => tvhr,
      wb_tm_tai_h_ld_i => tvhr_ld,

      tm_cycles_o      => tm_cycles,
      tm_tai_o         => tm_tai
    );

  --============================================================================
  -- Clock and reset generation
  --============================================================================
  p_clk_20 : process
  begin
    clk_20 <= '0';
    wait for c_clk_20_per/2;
    clk_20 <= '1';
    wait for c_clk_20_per/2;
  end process p_clk_20;

  p_clk_125 : process
  begin
    clk_125 <= '0';
    wait for c_clk_125_per/2;
    clk_125 <= '1';
    wait for c_clk_125_per/2;
  end process p_clk_125;

  p_rst_20_n : process
  begin
    rst_20_n <= '0';
    wait for c_reset_width;
    wait for c_clk_20_per;
    rst_20_n <= '1';
    wait;
  end process p_rst_20_n;

  cmp_sync_rst : gc_sync_ffs
    port map
    (
      clk_i    => clk_125,
      rst_n_i  => '1',
      data_i   => rst_20_n,
      synced_o => rst_125_n
    );

  --============================================================================
  -- Stimuli
  --============================================================================
  -- Stimuli process
  p_stim : process
  begin
    l <= (others => '0');
    h <= (others => '0');
    ldh <= '0';
    ldl <= '0';
    wait for 1 us;
    l <= x"0000f000";
    ldl <= '1';
    wait for c_clk_20_per;
    ldl <= '0';
    wait for 1 us;
    h <= x"01";
    ldh <= '1';
    wait for c_clk_20_per;
    ldh <= '0';
    wait;
  end process p_stim;

  -- Set the TVR on clock boundaries
  p_tvr : process (clk_20)
  begin
    if rising_edge(clk_20) then
      if rst_20_n = '0' then
        tvlr <= (others => '0');
        tvhr <= (others => '0');
        tvlr_ld <= '0';
        tvhr_ld <= '0';
      elsif ldl = '1' then
        tvlr <= l;
        tvlr_ld <= '1';
      elsif ldh = '1' then
        tvhr <= h;
        tvhr_ld <= '1';
      else
        tvlr_ld <= '0';
        tvhr_ld <= '0';
      end if;
    end if;
  end process p_tvr;

end architecture behav;
--==============================================================================
--  architecture end
--==============================================================================
